/*
 * version.h --- controls the version number printed by the e2fs
 * programs.
 *
 * Copyright 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
 * 2004, 2005, 2006, 2007, 2008, 2009, 2010 by Theodore Ts'o.  This
 * file may be redistributed under the GNU Public License v2.
 */

#define E2FSPROGS_VERSION "1.43.6"
#define E2FSPROGS_DATE "29-Aug-2017"
